package com.sun.corba.se.spi.servicecontext;

import java.util.Iterator;
import com.sun.corba.se.impl.util.Utility;
import com.sun.corba.se.impl.encoding.EncapsInputStream;
import org.omg.CORBA.CompletionStatus;
import sun.corba.EncapsInputStreamFactory;
import com.sun.corba.se.impl.encoding.CDRInputStream;
import java.util.HashMap;
import org.omg.CORBA.OctetSeqHelper;
import com.sun.corba.se.impl.orbutil.ORBUtility;
import org.omg.CORBA_2_3.portable.InputStream;
import org.omg.CORBA_2_3.portable.OutputStream;
import com.sun.corba.se.impl.logging.ORBUtilSystemException;
import com.sun.corba.se.spi.ior.iiop.GIOPVersion;
import com.sun.org.omg.SendingContext.CodeBase;
import java.util.Map;
import com.sun.corba.se.spi.orb.ORB;

public class ServiceContexts
{
    private static final int JAVAIDL_ALIGN_SERVICE_ID = -1106033203;
    private ORB orb;
    private Map scMap;
    private boolean addAlignmentOnWrite;
    private CodeBase codeBase;
    private GIOPVersion giopVersion;
    private ORBUtilSystemException wrapper;
    
    private static boolean isDebugging(OutputStream outputStream) {
        ORB orb = (ORB)(ORB)outputStream.orb();
        return orb != null && orb.serviceContextDebugFlag;
    }
    
    private static boolean isDebugging(InputStream inputStream) {
        ORB orb = (ORB)(ORB)inputStream.orb();
        return orb != null && orb.serviceContextDebugFlag;
    }
    
    private void dprint(String s) {
        ORBUtility.dprint(this, s);
    }
    
    public static void writeNullServiceContext(OutputStream outputStream) {
        if (isDebugging(outputStream)) {
            ORBUtility.dprint("ServiceContexts", "Writing null service context");
        }
        outputStream.write_long(0);
    }
    
    private void createMapFromInputStream(InputStream inputStream) {
        this.orb = (ORB)(ORB)inputStream.orb();
        if (this.orb.serviceContextDebugFlag) {
            this.dprint("Constructing ServiceContexts from input stream");
        }
        int read_long = inputStream.read_long();
        if (this.orb.serviceContextDebugFlag) {
            this.dprint("Number of service contexts = " + read_long);
        }
        for (int i = 0; i < read_long; ++i) {
            int read_long2 = inputStream.read_long();
            if (this.orb.serviceContextDebugFlag) {
                this.dprint("Reading service context id " + read_long2);
            }
            byte[] read = OctetSeqHelper.read(inputStream);
            if (this.orb.serviceContextDebugFlag) {
                this.dprint("Service context" + read_long2 + " length: " + read.length);
            }
            this.scMap.put(new Integer(read_long2), read);
        }
    }
    
    public ServiceContexts(ORB orb) {
        this.orb = orb;
        this.wrapper = ORBUtilSystemException.get(orb, "rpc.protocol");
        this.addAlignmentOnWrite = false;
        this.scMap = new HashMap();
        this.giopVersion = orb.getORBData().getGIOPVersion();
        this.codeBase = null;
    }
    
    public ServiceContexts(InputStream inputStream) {
        this((ORB)(ORB)inputStream.orb());
        this.codeBase = ((CDRInputStream)inputStream).getCodeBase();
        this.createMapFromInputStream(inputStream);
        this.giopVersion = ((CDRInputStream)inputStream).getGIOPVersion();
    }
    
    private ServiceContext unmarshal(Integer obj, byte[] array) {
        ServiceContextData serviceContextData = this.orb.getServiceContextRegistry().findServiceContextData((int)obj);
        UnknownServiceContext serviceContext;
        if (serviceContextData == null) {
            if (this.orb.serviceContextDebugFlag) {
                this.dprint("Could not find ServiceContextData for " + obj + " using UnknownServiceContext");
            }
            serviceContext = new UnknownServiceContext((int)obj, array);
        }
        else {
            if (this.orb.serviceContextDebugFlag) {
                this.dprint("Found " + serviceContextData);
            }
            EncapsInputStream encapsInputStream = EncapsInputStreamFactory.newEncapsInputStream(this.orb, array, array.length, this.giopVersion, this.codeBase);
            encapsInputStream.consumeEndian();
            serviceContext = serviceContextData.makeServiceContext(encapsInputStream, this.giopVersion);
            if (serviceContext == null) {
                throw this.wrapper.svcctxUnmarshalError(CompletionStatus.COMPLETED_MAYBE);
            }
        }
        return serviceContext;
    }
    
    public void addAlignmentPadding() {
        this.addAlignmentOnWrite = true;
    }
    
    public void write(OutputStream outputStream, GIOPVersion giopVersion) {
        if (isDebugging(outputStream)) {
            this.dprint("Writing service contexts to output stream");
            Utility.printStackTrace();
        }
        int size = this.scMap.size();
        if (this.addAlignmentOnWrite) {
            if (isDebugging(outputStream)) {
                this.dprint("Adding alignment padding");
            }
            ++size;
        }
        if (isDebugging(outputStream)) {
            this.dprint("Service context has " + size + " components");
        }
        outputStream.write_long(size);
        this.writeServiceContextsInOrder(outputStream, giopVersion);
        if (this.addAlignmentOnWrite) {
            if (isDebugging(outputStream)) {
                this.dprint("Writing alignment padding");
            }
            outputStream.write_long(-1106033203);
            outputStream.write_long(4);
            outputStream.write_octet((byte)0);
            outputStream.write_octet((byte)0);
            outputStream.write_octet((byte)0);
            outputStream.write_octet((byte)0);
        }
        if (isDebugging(outputStream)) {
            this.dprint("Service context writing complete");
        }
    }
    
    private void writeServiceContextsInOrder(OutputStream outputStream, GIOPVersion giopVersion) {
        Integer n = new Integer(9);
        Object remove = this.scMap.remove(n);
        for (Integer n2 : this.scMap.keySet()) {
            this.writeMapEntry(outputStream, n2, this.scMap.get(n2), giopVersion);
        }
        if (remove != null) {
            this.writeMapEntry(outputStream, n, remove, giopVersion);
            this.scMap.put(n, remove);
        }
    }
    
    private void writeMapEntry(OutputStream outputStream, Integer obj, Object o, GIOPVersion giopVersion) {
        if ((o instanceof byte[])) {
            if (isDebugging(outputStream)) {
                this.dprint("Writing service context bytes for id " + obj);
            }
            OctetSeqHelper.write(outputStream, (byte[])(byte[])o);
        }
        else {
            ServiceContext obj2 = (ServiceContext)o;
            if (isDebugging(outputStream)) {
                this.dprint("Writing service context " + obj2);
            }
            obj2.write(outputStream, giopVersion);
        }
    }
    
    public void put(ServiceContext serviceContext) {
        this.scMap.put(new Integer(serviceContext.getId()), serviceContext);
    }
    
    public void delete(int value) {
        this.delete(new Integer(value));
    }
    
    public void delete(Integer n) {
        this.scMap.remove(n);
    }
    
    public ServiceContext get(int value) {
        return this.get(new Integer(value));
    }
    
    public ServiceContext get(Integer n) {
        Object value = this.scMap.get(n);
        if (value == null) {
            return null;
        }
        if ((value instanceof byte[])) {
            ServiceContext unmarshal = this.unmarshal(n, (byte[])(byte[])value);
            this.scMap.put(n, unmarshal);
            return unmarshal;
        }
        return (ServiceContext)value;
    }
}
